@@ -28,6 +28,7 @@
 #include "literal.h"
 
 #include <QtCore/QTimer>
+#include <QtCore/QFile>
 
 #include <Soprano/Model>
 #include <Soprano/QueryResultIterator>
@@ -388,6 +389,160 @@ void DuplicateStatementJob::execute()
     }
 }
 
+class InvalidFileResourcesJob : public CleaningJob {
+public:
+    explicit InvalidFileResourcesJob(QObject* parent = 0)
+    : CleaningJob(parent) {}
+
+    virtual QString jobName() {
+        return i18n("Cleaning invalid file metadata");
+    }
+private:
+    virtual void execute();
+};
+
+void InvalidFileResourcesJob::execute()
+{
+    //
+    // Delete all the files that do not have a url
+    //
+    QLatin1String query("select distinct ?r where { ?r a nfo:FileDataObject. FILTER NOT EXISTS {"
+                        " ?r nie:url ?url . }");
+
+    Soprano::Model *model = Nepomuk2::ResourceManager::instance()->mainModel();
+    Soprano::QueryResultIterator it = model->executeQuery( query, Soprano::Query::QueryLanguageSparql );
+
+    QList<QUrl> deleteList;
+    while( it.next() && !shouldQuit() ) {
+        deleteList << it[0].uri();
+
+        if( deleteList.size() > 10 ) {
+            KJob* job = Nepomuk2::removeResources( deleteList );
+            job->exec();
+            deleteList.clear();
+        }
+    }
+
+    if( !deleteList.isEmpty() ) {
+        KJob* job = Nepomuk2::removeResources( deleteList );
+        job->exec();
+        deleteList.clear();
+    }
+
+    //
+    // Delete the files whose url does not exist
+    //
+
+    query = QLatin1String( "select distinct ?r ?url where { "
+                           "?r a nfo:FileDataObject ; nie:url ?url . }" );
+    it = model->executeQuery( query, Soprano::Query::QueryLanguageSparql );
+
+    while( it.next() && !shouldQuit() ) {
+        QUrl url( it["url"].uri() );
+        QString file = url.toLocalFile();
+
+        if( !file.isEmpty() && !QFile::exists(file) ) {
+            deleteList << it["r"].uri();
+        }
+
+        if( deleteList.size() > 10 ) {
+            KJob* job = Nepomuk2::removeResources( deleteList );
+            job->exec();
+            deleteList.clear();
+        }
+    }
+
+    if( !deleteList.isEmpty() ) {
+        KJob* job = Nepomuk2::removeResources( deleteList );
+        job->exec();
+        deleteList.clear();
+    }
+}
+
+
+class InvalidResourcesJob : public CleaningJob {
+public:
+    explicit InvalidResourcesJob(QObject* parent = 0)
+    : CleaningJob(parent) {}
+
+    virtual QString jobName() {
+        return i18n("Cleaning invalid resources");
+    }
+private:
+    virtual void execute();
+};
+
+void InvalidResourcesJob::execute()
+{
+    // Clear all the resources which do not have any rdf:type
+    QLatin1String query("select distinct ?r where { ?r ?p ?o . FILTER NOT EXISTS { ?r a ?t . }");
+
+    Soprano::Model *model = Nepomuk2::ResourceManager::instance()->mainModel();
+    Soprano::QueryResultIterator it = model->executeQuery( query, Soprano::Query::QueryLanguageSparqlNoInference );
+
+    QList<QUrl> deleteList;
+    while( it.next() && !shouldQuit() ) {
+        deleteList << it[0].uri();
+
+        if( deleteList.size() > 10 ) {
+            KJob* job = Nepomuk2::removeResources( deleteList );
+            job->exec();
+            deleteList.clear();
+        }
+    }
+
+    if( !deleteList.isEmpty() ) {
+        KJob* job = Nepomuk2::removeResources( deleteList );
+        job->exec();
+        deleteList.clear();
+    }
+}
+
+class InvalidStatementsJob : public CleaningJob {
+public:
+    explicit InvalidStatementsJob(QObject* parent = 0)
+    : CleaningJob(parent) {}
+
+    virtual QString jobName() {
+        return i18n("Cleaning invalid statements");
+    }
+private:
+    virtual void execute();
+};
+
+void InvalidStatementsJob::execute()
+{
+    // Clear all the statements which are violating the nrl:maxCardinality
+    QString query = QString::fromLatin1("select distinct ?r ?p  where { ?p nrl:maxCardinality %1 ."
+                                        " ?r ?p ?o1 , ?o2 . FILTER( ?o1 != ?o2 ) . }")
+                    .arg( Soprano::Node::literalToN3( Soprano::LiteralValue(1) ) );
+
+    Soprano::Model *model = Nepomuk2::ResourceManager::instance()->mainModel();
+    Soprano::QueryResultIterator it = model->executeQuery( query, Soprano::Query::QueryLanguageSparqlNoInference );
+
+    while( it.next() && !shouldQuit() ) {
+        Soprano::Statement st( it[0], it[1], Soprano::Node() );
+        kDebug() << st;
+
+        // Keep the oldest statement
+        QString query = QString::fromLatin1("select ?o ?g where { graph ?g { %1 %2 ?o . } "
+                                            "?g nao:created ?c . } ORDER BY ASC(?c) LIMIT 1")
+                        .arg( st.subject().toN3(), st.predicate().toN3() );
+        Soprano::QueryResultIterator iter = model->executeQuery( query, Soprano::Query::QueryLanguageSparqlNoInference );
+        if( iter.next() ) {
+            st.setObject( iter[0] );
+            st.setContext( iter[1] );
+        }
+
+        // FIXME: What about the trailing graphs?
+        kDebug() << st;
+        model->removeAllStatements( st.subject(), st.predicate(), Soprano::Node() );
+        model->addStatement( st );
+    }
+
+    // TODO: Figure out how to clear the statements which do not follow the domain and range
+    //       restrictions
+}
 
 
 QList< CleaningJob* > allJobs()
@@ -401,7 +556,10 @@ QList< CleaningJob* > allJobs()
     list << new AkonadiMigrationJob();
     list << new DuplicateStatementJob();
     list << new DuplicateContactJob();
+    list << new InvalidFileResourcesJob();
+    list << new InvalidResourcesJob();
+    list << new InvalidStatementsJob();
     return list;
 }
 
-#include "cleaningjobs.moc"
\ No newline at end of file
+#include "cleaningjobs.moc"
