@@ -449,24 +449,12 @@ QString Nepomuk2::Query::Query::toSparqlQuery( SparqlFlags sparqlFlags ) const
     // actually build the SPARQL query patterns
     QueryBuilderData qbd( d.constData(), sparqlFlags );
 
-
-    //
-    // We restrict results to user visible types only. There is no need for this with file queries as normally all files are visible.
-    //
-    QString userVisibilityRestriction;
-    if( !(queryFlags()&NoResultRestrictions) && !d->m_isFileQuery ) {
-        userVisibilityRestriction = QString::fromLatin1("FILTER EXISTS { ?r a [ %1 %2 ] . } . ")
-                                    .arg(Soprano::Node::resourceToN3(Soprano::Vocabulary::NAO::userVisible()),
-                                         Soprano::Node::literalToN3(Soprano::LiteralValue(true)));
-    }
-
-
     if(!term.isValid()) {
         return QString();
     }
 
     QString termGraphPattern;
-    termGraphPattern = term.d_ptr->toSparqlGraphPattern( QLatin1String( "?r" ), 0, userVisibilityRestriction, &qbd );
+    termGraphPattern = term.d_ptr->toSparqlGraphPattern( QLatin1String( "?r" ), 0, QString(), &qbd );
     if( termGraphPattern.isEmpty() ) {
         kDebug() << "Got no valid SPARQL pattern from" << term;
         return QString();
@@ -528,6 +516,23 @@ QString Nepomuk2::Query::Query::toSparqlQuery( SparqlFlags sparqlFlags ) const
     if ( d->m_limit > 0 )
         query += QString::fromLatin1( " LIMIT %1" ).arg( d->m_limit );
 
+    // We never want to show ontology data to the users
+    // FIXME: This has been causing virtuoso to crash on a large number of queries
+    // For now I'm disabling this. We might need to find a better solution. This just sucks.
+    //
+    // Example query (which causes the crash) -
+    //
+    // sparql DEFINE input:inference <nepomukinference>
+    //        define input:default-graph-exclude <nepomuk-ontology-group>
+    //        select distinct ?r ?reqProp1 where {
+    //        { ?r nie:isPartOf <nepomuk:/res/d5cb1ed7-a80b-46b6-8467-35716f5e7188> .
+    //          ?r nmo:plainTextMessageContent ?v1 .
+    //          FILTER(bif:contains(?v1, "'funcom'")) .
+    //          OPTIONAL { ?r <http://akonadi-project.org/ontologies/aneo#akonadiItemId> ?reqProp1 . }
+    //        } .
+    //  }
+    // query = QLatin1String("define input:default-graph-exclude <nepomuk-ontology-group> ") + query;
+
     return query.simplified();
 }
 
