@@ -1,7 +1,7 @@
 /*
  * This file is part of the Nepomuk KDE project.
  * Copyright (C) 2006-2012 Sebastian Trueg <trueg@kde.org>
- * Copyright (C) 2010 Vishesh Handa <handa.vish@gmail.com>
+ * Copyright (C) 2010-2012 Vishesh Handa <handa.vish@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -61,7 +61,6 @@ Nepomuk2::ResourceManagerPrivate::ResourceManagerPrivate( ResourceManager* manag
     : mainModel( 0 ),
       overrideModel( 0 ),
       mutex(QMutex::Recursive),
-      dataCnt( 0 ),
       m_manager( manager ),
       m_watcher( 0 )
 {
@@ -229,6 +228,25 @@ Nepomuk2::ResourceManager::ResourceManager()
 
 }
 
+void Nepomuk2::ResourceManager::cleanupResources()
+{
+    //
+    // Ideally, all three caches should be empty when the ResourceManager is being destroyed
+    // But that isn't always the case. There could be a Resource in a static object which gets
+    // deleted after the ResourceManager.
+    // In order to counter that case, we mark all Resource classes as invalid.
+    // The cache should be empty when the ResourceManager is getting destroyed
+    // See bug 292996 - https://bugs.kde.org/show_bug.cgi?id=292996
+    //
+    QMutexLocker lock( &d->mutex );
+    QList<ResourceData*> rdList = d->m_identifierKickOff.values() + d->m_urlKickOff.values() +
+                                  d->m_initializedData.values();
+    foreach( const ResourceData* rd, rdList.toSet() ) {
+        foreach(Resource* res, rd->resources())
+            res->m_data = 0;
+        delete rd;
+    }
+}
 
 Nepomuk2::ResourceManager::~ResourceManager()
 {
@@ -249,8 +267,16 @@ Nepomuk2::ResourceManager::~ResourceManager()
 Nepomuk2::ResourceManager* Nepomuk2::ResourceManager::instance()
 {
     if(!s_instance) {
+        //
+        // This happens when QCoreApplication::quit has been called
+        //
+        QCoreApplication* app = QCoreApplication::instance();
+        if( !app )
+            return 0;
+
         s_instance = new ResourceManager();
-        s_instance->setParent(QCoreApplication::instance());
+        s_instance->setParent( app );
+        connect( app, SIGNAL(aboutToQuit()), s_instance, SLOT(cleanupResources()) );
     }
     return s_instance;
 }
@@ -312,18 +338,20 @@ QUrl Nepomuk2::ResourceManager::generateUniqueUri( const QString& name )
     while( 1 ) {
         QString uuid = QUuid::createUuid().toString();
         uuid = uuid.mid(1, uuid.length()-2);
-        QUrl uri = QUrl( QLatin1String("nepomuk:/") + type + QLatin1String("/") + uuid );
-        if ( !model->executeQuery( QString::fromLatin1("ask where { "
-                                                       "{ <%1> ?p1 ?o1 . } "
-                                                       "UNION "
-                                                       "{ ?s2 <%1> ?o2 . } "
-                                                       "UNION "
-                                                       "{ ?s3 ?p3 <%1> . } "
-                                                       "UNION "
-                                                       "{ graph <%1> { ?s4 ?4 ?o4 . } . } "
-                                                       "}")
-                                   .arg( QString::fromAscii( uri.toEncoded() ) ), Soprano::Query::QueryLanguageSparql ).boolValue() ) {
-            return uri;
+        QString uriString = QLatin1String("nepomuk:/") + type + QLatin1String("/") + uuid;
+        QString query = QString::fromLatin1("select iri_to_id( '%1', 0 )").arg( uriString );
+
+        Soprano::QueryResultIterator it = model->executeQuery( query, Soprano::Query::QueryLanguageUser,
+                                                               QLatin1String("sql") );
+
+        // don't loop forever if there was an error executing the query
+        if( model->lastError() ) {
+            return QUrl();
+        }
+        if( it.next() ) {
+            if( it[0].literal().toString().isEmpty() ) {
+                return QUrl( uriString );
+            }
         }
     }
 }
@@ -376,4 +404,48 @@ void Nepomuk2::ResourceManager::setOverrideMainModel( Soprano::Model* model )
         }
     }
 }
+
+void Nepomuk2::ResourceManagerPrivate::addToWatcher( const QUrl& uri )
+{
+    if( uri.isEmpty() )
+        return;
+
+    if( !m_watcher ) {
+        m_watcher = new ResourceWatcher(m_manager);
+        //
+        // The ResourceWatcher is not thread-safe. Thus, we need to ensure the safety ourselves.
+        // We do that by simply handling all RW related operations in the manager thread.
+        // This also means to invoke methods on the watcher through QMetaObject to make sure they
+        // get queued in case of calls between different threads.
+        //
+        m_watcher->moveToThread(m_manager->thread());
+        QObject::connect( m_watcher, SIGNAL(propertyAdded(Nepomuk2::Resource, Nepomuk2::Types::Property, QVariant)),
+                          m_manager, SLOT(slotPropertyAdded(Nepomuk2::Resource, Nepomuk2::Types::Property, QVariant)) );
+        QObject::connect( m_watcher, SIGNAL(propertyRemoved(Nepomuk2::Resource, Nepomuk2::Types::Property, QVariant)),
+                          m_manager, SLOT(slotPropertyRemoved(Nepomuk2::Resource, Nepomuk2::Types::Property, QVariant)) );
+        m_watcher->addResource( uri );
+    }
+    else {
+        QMetaObject::invokeMethod( m_watcher, "addResource", Qt::AutoConnection, Q_ARG(QUrl, uri) );
+    }
+    // (re-)start the watcher in case this resource is the only one in the list of watched
+    if( m_watcher->resourceCount() <= 1 ) {
+        QMetaObject::invokeMethod(m_watcher, "start", Qt::AutoConnection);
+    }
+}
+
+void Nepomuk2::ResourceManagerPrivate::removeFromWatcher( const QUrl& uri )
+{
+    if( uri.isEmpty() || !m_watcher )
+        return;
+
+    // stop the watcher since we do not want to watch all changes in case there is no ResourceData left
+    if(m_watcher->resourceCount() == 1) {
+        QMetaObject::invokeMethod(m_watcher, "stop", Qt::AutoConnection);
+    }
+
+    // remove this Resource from the list of watched resources
+    QMetaObject::invokeMethod(m_watcher, "removeResource", Qt::AutoConnection, Q_ARG(QUrl, uri));
+}
+
 #include "resourcemanager.moc"
