@@ -16,13 +16,13 @@
    Boston, MA 02110-1301, USA.
 */
 
-#ifndef _NEPOMUK_STRIGI_SERVICE_CONFIG_H_
-#define _NEPOMUK_STRIGI_SERVICE_CONFIG_H_
+#ifndef _NEPOMUK_FILEINDEXER_SERVICE_CONFIG_H_
+#define _NEPOMUK_FILEINDEXER_SERVICE_CONFIG_H_
 
 #include <QtCore/QObject>
 #include <QtCore/QList>
 #include <QtCore/QRegExp>
-#include <QtCore/QMutex>
+#include <QReadWriteLock>
 
 #include <kconfig.h>
 #include <kio/global.h>
@@ -71,11 +71,6 @@ namespace Nepomuk2 {
 
         QStringList excludeFilters() const;
 
-        /**
-         * Strigi plugins not to call.
-         */
-	QStringList excludePlugins() const;
-
         bool indexHiddenFilesAndFolders() const;
 
         /**
@@ -144,6 +139,19 @@ namespace Nepomuk2 {
          */
         bool shouldFileBeIndexed( const QString& fileName ) const;
 
+        /**
+         * Check whether \p path should be watched. Most folders need
+         * watches installed on them, even if they are not indexed,
+         * otherwise moving files into a non-indexed directory would
+         * cause their metadata to be lost.
+         *
+         * This function does not check whether a function should be indexed.
+         * Since every indexed folder should be watched, this function should
+         * only be called on a non-indexed folder.
+         *
+         * \return \p true always for now.
+         */
+        bool shouldFolderBeWatched( const QString& path ) const;
         /**
          * Check if the debug mode is enabled. The debug mode is a hidden
          * configuration option (without any GUI) that will make the indexer
@@ -151,12 +159,6 @@ namespace Nepomuk2 {
          */
         bool isDebugModeEnabled() const;
 
-        /**
-         * Returns the libstreamanalyzer version which was last used when all
-         * the files were checked, and indexed
-         */
-        QString strigiVersion() const;
-
     Q_SIGNALS:
         void configChanged();
 
@@ -174,13 +176,6 @@ namespace Nepomuk2 {
          */
         void setInitialRun(bool isInitialRun);
 
-        /**
-         * Should be called once all the files have been indexed. This is done in
-         * order to avoid rechecking all the files if the strigi version has not
-         * changed.
-         */
-        void setStrigiVersion(const QString& version);
-
     private Q_SLOTS:
         void slotConfigDirty();
 
@@ -203,7 +198,7 @@ namespace Nepomuk2 {
         /// to prevent regexp parsing over and over
         RegExpCache m_excludeFilterRegExpCache;
 
-        mutable QMutex m_folderCacheMutex;
+        mutable QReadWriteLock m_folderCacheMutex;
 
         static FileIndexerConfig* s_self;
     };
