@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2007-2010 Sebastian Trueg <trueg@kde.org>
+   Copyright (C) 2012 Vishesh Handa <me@vhanda.in>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,13 +19,14 @@
 */
 
 #include "kinotify.h"
+#include "optimizedbytearray.h"
 
 #include <QtCore/QSocketNotifier>
 #include <QtCore/QHash>
 #include <QtCore/QDirIterator>
 #include <QtCore/QFile>
-#include <QtCore/QQueue>
 #include <QtCore/QScopedArrayPointer>
+#include <QtCore/QLinkedList>
 
 #include <kdebug.h>
 
@@ -75,11 +77,15 @@ public:
     }
 
     QHash<int, QByteArray> cookies;
-    QHash<int, QByteArray> watchPathHash;
-    QHash<QByteArray, int> pathWatchHash;
+
+    // url <-> wd mappings
+    // Read the documentation fo OptimizedByteArray to understand why have a cache
+    QHash<int, OptimizedByteArray> watchPathHash;
+    QHash<OptimizedByteArray, int> pathWatchHash;
+    QSet<QByteArray> pathCache;
 
     /// A list of all the current dirIterators
-    QQueue<QDirIterator*> dirIterators;
+    QLinkedList<QDirIterator*> dirIterators;
 
     unsigned char eventBuffer[EVENT_BUFFER_SIZE];
 
@@ -108,21 +114,15 @@ public:
         WatchFlags newFlags = flags;
 
         if( !q->filterWatch( path, newMode, newFlags ) ) {
-            return true;
+            return false;
         }
         // we always need the unmount event to maintain our path hash
         const int mask = newMode|newFlags|EventUnmount|FlagExclUnlink;
 
-        return addWatchNoCheck(path, mask);
-    }
-
-    /*This function adds a watch assuming the filtering has already been done,
-     * so it can be used in both addWatch and addWatchRecursive */
-    bool addWatchNoCheck( const QByteArray& path, const int mask ) {
         int wd = inotify_add_watch( inotify(), path.data(), mask );
         if ( wd > 0 ) {
 //            kDebug() << "Successfully added watch for" << path << pathHash.count();
-            QByteArray normalized = stripTrailingSlash( path );
+            OptimizedByteArray normalized( stripTrailingSlash( path ), pathCache );
             watchPathHash.insert( wd, normalized );
             pathWatchHash.insert( normalized, wd );
             return true;
@@ -140,72 +140,29 @@ public:
     }
 
     void removeWatch( int wd ) {
-        kDebug() << wd << watchPathHash[wd];
+        kDebug() << wd << watchPathHash[wd].toByteArray();
         pathWatchHash.remove( watchPathHash.take( wd ) );
         inotify_rm_watch( inotify(), wd );
     }
 
-    /*Number of watches to add in one go, before deferring the rest to a future invocation*/
-    #define WATCHES_AT_ONCE 200
-
-    /*Add watches recursively. We don't use the QDir::Subfolders attribute
-     * because we want to avoid walking the directory tree in subfolders we aren't indexing:
-     * they may be large, complicated and mounted over a network.*/
-    bool addWatchesRecursively(const QString& path, int& count){
-        WatchEvents newMode = mode;
-        WatchFlags newFlags = flags;
-        //If we do not want to watch this directory, go home, reporting success.
-        if(!q->filterWatch( path, newMode, newFlags )){
-            return true;
-        }
-
-        // We always need the unmount event to maintain our path hash
-        const int mask = newMode|newFlags|EventUnmount|FlagExclUnlink;
-        //Try to install a watch on this directory.
-        if(!addWatchNoCheck( QFile::encodeName(path) , mask )){
-            return false;
-        }
-        //We installed a watch, so increment our counter.
-        count++;
-        //Next try to add subdirectories, calling self recursively.
-        QDirIterator* it = new QDirIterator( path, QDir::Dirs | QDir::NoDotAndDotDot);
-        //If we have added enough directories already,
-        //queue any subdirectories in this one for later addition
-        //and stop.
-        if(count > WATCHES_AT_ONCE+25){
-            if(it->hasNext()){
-                dirIterators.append(it);
-            }
-            return true;
-        }
-        //Note that because we check count before this, if count=99 here,
-        //we may add all top-level subdirectories to the queue.
-        while(it->hasNext()){
-            if(!addWatchesRecursively(it->next(),count)){
-                return false;
-            }
-        }
-        return true;
-    }
-
+    /**
+     * Add one watch and call oneself asynchronously
+     */
     bool _k_addWatches() {
-        /* add the next batch of paths
-         * i is incrememented inside addWatchesRecursively
-         * Note that we allow i to become a bit larger than WATCHES_AT_ONCE
-         * in addWatchesRecursively, to starting a new tree near the end of a batch.
-         */
-        for ( int i = 0; i < WATCHES_AT_ONCE && !dirIterators.isEmpty(); ) {
+        bool addedWatchSuccessfully = false;
+
+        if( !dirIterators.isEmpty() ) {
             QDirIterator* it = dirIterators.front();
             if( it->hasNext() ) {
-                it->next();
-                //Try to recursively install watches on this directory.
-                if( !addWatchesRecursively(it->filePath(),i) ){
-                    return false;
+                QString dirPath = it->next();
+                if( addWatch( QFile::encodeName(dirPath) ) ) {
+                    QDirIterator* iter= new QDirIterator( dirPath, QDir::Dirs | QDir::NoDotAndDotDot);
+                    dirIterators.push_front( iter );
+                    addedWatchSuccessfully = true;
                 }
             }
             else {
-                delete dirIterators.dequeue();
-                break;
+                delete dirIterators.takeFirst();
             }
         }
 
@@ -213,7 +170,8 @@ public:
         if ( !dirIterators.isEmpty() ) {
             QMetaObject::invokeMethod( q, "_k_addWatches", Qt::QueuedConnection );
         }
-	return true;
+
+        return addedWatchSuccessfully;
     }
 
 private:
@@ -279,7 +237,7 @@ bool KInotify::available() const
 bool KInotify::watchingPath( const QString& path ) const
 {
     const QByteArray p( stripTrailingSlash( QFile::encodeName( path ) ) );
-    return d->pathWatchHash.contains(p);
+    return d->pathWatchHash.contains( OptimizedByteArray(p, d->pathCache) );
 }
 
 
@@ -299,11 +257,21 @@ bool KInotify::addWatch( const QString& path, WatchEvents mode, WatchFlags flags
 
 bool KInotify::removeWatch( const QString& path )
 {
-    kDebug() << path;
-    QByteArray encodedPath = QFile::encodeName( path );
-    QHash<int, QByteArray>::iterator it = d->watchPathHash.begin();
+    // Stop all of the dirIterators which contain path
+    QMutableLinkedListIterator<QDirIterator*> iter( d->dirIterators );
+    while( iter.hasNext() ) {
+        QDirIterator* dirIter = iter.next();
+        if( dirIter->filePath().startsWith( path ) ) {
+            iter.remove();
+            delete dirIter;
+        }
+    }
+
+    // Remove all the watches
+    QByteArray encodedPath( QFile::encodeName( path ) );
+    QHash<int, OptimizedByteArray>::iterator it = d->watchPathHash.begin();
     while ( it != d->watchPathHash.end() ) {
-        if ( it.value().startsWith( encodedPath ) ) {
+        if ( it.value().toByteArray().startsWith( encodedPath ) ) {
             inotify_rm_watch( d->inotify(), it.key() );
             d->pathWatchHash.remove(it.value());
             it = d->watchPathHash.erase( it );
@@ -338,12 +306,12 @@ void KInotify::slotEvent( int socket )
         // the event name only contains an interesting value if we get an event for a file/folder inside
         // a watched folder. Otherwise we should ignore it
         if ( event->mask & (EventDeleteSelf|EventMoveSelf) ) {
-            path = d->watchPathHash.value( event->wd );
+            path = d->watchPathHash.value( event->wd ).toByteArray();
         }
         else {
             // we cannot use event->len here since it contains the size of the buffer and not the length of the string
             const QByteArray eventName = QByteArray::fromRawData( event->name, qstrnlen(event->name,event->len) );
-            const QByteArray hashedPath = d->watchPathHash.value( event->wd );
+            const QByteArray hashedPath = d->watchPathHash.value( event->wd ).toByteArray();
             path = concatPath( hashedPath, eventName );
         }
 
@@ -405,13 +373,15 @@ void KInotify::slotEvent( int socket )
 
                 // update the path cache
                 if( event->mask & IN_ISDIR ) {
-                    QHash<QByteArray, int>::iterator it = d->pathWatchHash.find(oldPath);
+                    OptimizedByteArray optimOldPath( oldPath, d->pathCache );
+                    QHash<OptimizedByteArray, int>::iterator it = d->pathWatchHash.find( optimOldPath );
                     if( it != d->pathWatchHash.end() ) {
                         kDebug() << oldPath << path;
                         const int wd = it.value();
-                        d->watchPathHash[wd] = path;
+                        OptimizedByteArray optimPath( path, d->pathCache );
+                        d->watchPathHash[wd] = optimPath;
                         d->pathWatchHash.erase(it);
-                        d->pathWatchHash.insert( path, wd );
+                        d->pathWatchHash.insert( optimPath, wd );
                     }
                 }
 //                kDebug() << oldPath << "EventMoveTo" << path;
