@@ -20,7 +20,6 @@
 #include "metadatamover.h"
 #include "fileindexerinterface.h"
 #include "fileexcludefilters.h"
-#include "invalidfileresourcecleaner.h"
 #include "removabledeviceindexnotification.h"
 #include "removablemediacache.h"
 #include "fileindexerconfig.h"
@@ -63,7 +62,6 @@ namespace {
         IgnoringKInotify( RegExpCache* rec, QObject* parent );
         ~IgnoringKInotify();
 
-        bool addWatch( const QString& path, WatchEvents modes, WatchFlags flags = WatchFlags() );
     protected:
         bool filterWatch( const QString & path, WatchEvents & modes, WatchFlags & flags );
 
@@ -81,20 +79,23 @@ namespace {
     {
     }
 
-    bool IgnoringKInotify::addWatch( const QString& path, WatchEvents modes, WatchFlags flags )
-    {
-        if( !m_pathExcludeRegExpCache->filenameMatch( path ) ) {
-            return KInotify::addWatch( path, modes, flags );
-        }
-        else {
-            kDebug() << "Ignoring watch patch" << path;
-            return false;
-        }
-    }
-
     bool IgnoringKInotify::filterWatch( const QString & path, WatchEvents & modes, WatchFlags & flags )
     {
         Q_UNUSED( flags );
+
+        // Some symlink handling.
+        QFileInfo info(path);
+        // Check whether this path is a symlink to a path which has already been watched.
+        // If so, do not watch it. This does not solve *all* double-watching;
+        // the symlink may be watched before the original path.
+        // However, it stops us trying to install infinite watches when, eg,
+        // someone has created a symlink to a parent directory.
+        if( info.isSymLink() && info.exists() ){
+            if( watchingPath( info.symLinkTarget() ) )
+                return false;
+        }
+        if( !Nepomuk2::FileIndexerConfig::self()->shouldFolderBeWatched( path ) )
+            return false;
         // Only watch the index folders for file creation and change.
         if( Nepomuk2::FileIndexerConfig::self()->shouldFolderBeIndexed( path ) ) {
             modes |= KInotify::EventCloseWrite;
@@ -180,8 +181,6 @@ Nepomuk2::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
             this, SLOT(slotDeviceTeardownRequested(const Nepomuk2::RemovableMediaCache::Entry*)));
     addWatchesForMountedRemovableMedia();
 
-    (new InvalidFileResourceCleaner(this))->start();
-
     connect( FileIndexerConfig::self(), SIGNAL( configChanged() ),
              this, SLOT( updateIndexedFoldersWatches() ) );
 }
@@ -228,7 +227,6 @@ void Nepomuk2::FileWatch::slotFilesDeleted( const QStringList& paths )
     }
 
     if( !urls.isEmpty() ) {
-        kDebug() << urls;
         m_metadataMover->removeFileMetadata( urls );
     }
 }
@@ -367,13 +365,6 @@ void Nepomuk2::FileWatch::addWatchesForMountedRemovableMedia()
 
 void Nepomuk2::FileWatch::slotDeviceMounted(const Nepomuk2::RemovableMediaCache::Entry* entry)
 {
-    //
-    // now that the device is mounted we can clean up our db - in case we have any
-    // data for file that have been deleted from the device in the meantime.
-    //
-    InvalidFileResourceCleaner* cleaner = new InvalidFileResourceCleaner(this);
-    cleaner->start(entry->mountPath());
-
     //
     // tell the file indexer to update the newly mounted device
     //
@@ -418,6 +409,7 @@ void Nepomuk2::FileWatch::slotDeviceTeardownRequested(const Nepomuk2::RemovableM
 {
 #ifdef BUILD_KINOTIFY
     if( m_dirWatch ) {
+        kDebug() << entry->mountPath();
         m_dirWatch->removeWatch( entry->mountPath() );
     }
 #endif
